import * as vue from 'vue';
import { Plugin, CSSProperties, HTMLAttributes, Ref, DefineComponent, PropType, ComputedRef, InjectionKey, VNode, UnwrapRef, UnwrapNestedRefs } from 'vue';
import { SandpackLogLevel, NpmRegistry, SandpackMessage, ListenerFunction, UnsubscribeFunction, BundlerState, SandpackError, SandpackBundlerFiles, SandpackClient, ReactDevToolsMode, SandpackMessageConsoleMethods, TestError } from '@codesandbox/sandpack-client';
import { Extension } from '@codemirror/state';
import { EditorView, KeyBinding } from '@codemirror/view';
import * as _stitches_core_types_styled_component from '@stitches/core/types/styled-component';
import * as _stitches_core_types_css_util from '@stitches/core/types/css-util';
import * as _stitches_core_types_config from '@stitches/core/types/config';

declare function SanpackPlugin(): Plugin;

/**
 * @hidden
 */
declare const ANGULAR_TEMPLATE: {
    files: {
        '/src/app/app.component.css': {
            code: string;
        };
        '/src/app/app.component.html': {
            code: string;
        };
        '/src/app/app.component.ts': {
            code: string;
        };
        '/src/app/app.module.ts': {
            code: string;
        };
        '/src/index.html': {
            code: string;
        };
        '/src/main.ts': {
            code: string;
        };
        '/src/polyfills.ts': {
            code: string;
        };
    };
    dependencies: {
        '@angular/core': string;
        '@angular/platform-browser': string;
        '@angular/platform-browser-dynamic': string;
        '@angular/common': string;
        '@angular/compiler': string;
        'zone.js': string;
        'core-js': string;
        rxjs: string;
    };
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const REACT_TEMPLATE: {
    files: {
        '/App.js': {
            code: string;
        };
        '/index.js': {
            code: string;
        };
        '/styles.css': {
            code: string;
        };
        '/public/index.html': {
            code: string;
        };
    };
    dependencies: {
        react: string;
        'react-dom': string;
        'react-scripts': string;
    };
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const REACT_TYPESCRIPT_TEMPLATE: {
    files: {
        'tsconfig.json': {
            code: string;
        };
        '/App.tsx': {
            code: string;
        };
        '/index.tsx': {
            code: string;
        };
        '/styles.css': {
            code: string;
        };
        '/public/index.html': {
            code: string;
        };
    };
    dependencies: {
        react: string;
        'react-dom': string;
        'react-scripts': string;
    };
    devDependencies: {
        '@types/react': string;
        '@types/react-dom': string;
        typescript: string;
    };
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const SOLID_TEMPLATE: {
    files: {
        '/App.tsx': {
            code: string;
        };
        '/index.tsx': {
            code: string;
        };
        '/styles.css': {
            code: string;
        };
        '/index.html': {
            code: string;
        };
    };
    dependencies: {
        'solid-js': string;
    };
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const SVELTE_TEMPLATE: {
    environment: string;
    main: string;
    entry: string;
    files: {
        '/App.svelte': {
            code: string;
        };
        '/index.js': {
            code: string;
        };
        '/public/index.html': {
            code: string;
        };
    };
    dependencies: {
        svelte: string;
    };
};

/**
 * @hidden
 */
declare const TEST_TYPESCRIPT_TEMPLATE: {
    files: {
        'tsconfig.json': {
            code: string;
        };
        '/add.ts': {
            code: string;
        };
        '/add.test.ts': {
            code: string;
        };
    };
    dependencies: {};
    devDependencies: {
        typescript: string;
    };
    entry: string;
    main: string;
    environment: string;
    mode: string;
};

/**
 * @hidden
 */
declare const VANILLA_TEMPLATE: {
    files: {
        '/src/index.js': {
            code: string;
        };
        '/src/styles.css': {
            code: string;
        };
        '/index.html': {
            code: string;
        };
    };
    dependencies: {};
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const VANILLA_TYPESCRIPT_TEMPLATE: {
    files: {
        'tsconfig.json': {
            code: string;
        };
        '/src/index.ts': {
            code: string;
        };
        '/src/styles.css': {
            code: string;
        };
        '/index.html': {
            code: string;
        };
    };
    dependencies: {};
    devDependencies: {
        typescript: string;
    };
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const VUE_TEMPLATE: {
    files: {
        '/src/App.vue': {
            code: string;
        };
        '/src/main.js': {
            code: string;
        };
        '/public/index.html': {
            code: string;
        };
    };
    dependencies: {
        vue: string;
        '@vue/cli-plugin-babel': string;
    };
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const VUE_TEMPLATE_3: {
    files: {
        '/src/App.vue': {
            code: string;
        };
        '/src/main.js': {
            code: string;
        };
        '/public/index.html': {
            code: string;
        };
    };
    dependencies: {
        'core-js': string;
        vue: string;
        '@vue/cli-plugin-babel': string;
    };
    entry: string;
    main: string;
    environment: string;
};

/**
 * @hidden
 */
declare const SANDBOX_TEMPLATES: {
    react: {
        files: {
            '/App.js': {
                code: string;
            };
            '/index.js': {
                code: string;
            };
            '/styles.css': {
                code: string;
            };
            '/public/index.html': {
                code: string;
            };
        };
        dependencies: {
            react: string;
            'react-dom': string;
            'react-scripts': string;
        };
        entry: string;
        main: string;
        environment: string;
    };
    'react-ts': {
        files: {
            'tsconfig.json': {
                code: string;
            };
            '/App.tsx': {
                code: string;
            };
            '/index.tsx': {
                code: string;
            };
            '/styles.css': {
                code: string;
            };
            '/public/index.html': {
                code: string;
            };
        };
        dependencies: {
            react: string;
            'react-dom': string;
            'react-scripts': string;
        };
        devDependencies: {
            '@types/react': string;
            '@types/react-dom': string;
            typescript: string;
        };
        entry: string;
        main: string;
        environment: string;
    };
    vue: {
        files: {
            '/src/App.vue': {
                code: string;
            };
            '/src/main.js': {
                code: string;
            };
            '/public/index.html': {
                code: string;
            };
        };
        dependencies: {
            vue: string;
            '@vue/cli-plugin-babel': string;
        };
        entry: string;
        main: string;
        environment: string;
    };
    vanilla: {
        files: {
            '/src/index.js': {
                code: string;
            };
            '/src/styles.css': {
                code: string;
            };
            '/index.html': {
                code: string;
            };
        };
        dependencies: {};
        entry: string;
        main: string;
        environment: string;
    };
    'vanilla-ts': {
        files: {
            'tsconfig.json': {
                code: string;
            };
            '/src/index.ts': {
                code: string;
            };
            '/src/styles.css': {
                code: string;
            };
            '/index.html': {
                code: string;
            };
        };
        dependencies: {};
        devDependencies: {
            typescript: string;
        };
        entry: string;
        main: string;
        environment: string;
    };
    vue3: {
        files: {
            '/src/App.vue': {
                code: string;
            };
            '/src/main.js': {
                code: string;
            };
            '/public/index.html': {
                code: string;
            };
        };
        dependencies: {
            'core-js': string;
            vue: string;
            '@vue/cli-plugin-babel': string;
        };
        entry: string;
        main: string;
        environment: string;
    };
    angular: {
        files: {
            '/src/app/app.component.css': {
                code: string;
            };
            '/src/app/app.component.html': {
                code: string;
            };
            '/src/app/app.component.ts': {
                code: string;
            };
            '/src/app/app.module.ts': {
                code: string;
            };
            '/src/index.html': {
                code: string;
            };
            '/src/main.ts': {
                code: string;
            };
            '/src/polyfills.ts': {
                code: string;
            };
        };
        dependencies: {
            '@angular/core': string;
            '@angular/platform-browser': string;
            '@angular/platform-browser-dynamic': string;
            '@angular/common': string;
            '@angular/compiler': string;
            'zone.js': string;
            'core-js': string;
            rxjs: string;
        };
        entry: string;
        main: string;
        environment: string;
    };
    svelte: {
        environment: string;
        main: string;
        entry: string;
        files: {
            '/App.svelte': {
                code: string;
            };
            '/index.js': {
                code: string;
            };
            '/public/index.html': {
                code: string;
            };
        };
        dependencies: {
            svelte: string;
        };
    };
    solid: {
        files: {
            '/App.tsx': {
                code: string;
            };
            '/index.tsx': {
                code: string;
            };
            '/styles.css': {
                code: string;
            };
            '/index.html': {
                code: string;
            };
        };
        dependencies: {
            'solid-js': string;
        };
        entry: string;
        main: string;
        environment: string;
    };
    'test-ts': {
        files: {
            'tsconfig.json': {
                code: string;
            };
            '/add.ts': {
                code: string;
            };
            '/add.test.ts': {
                code: string;
            };
        };
        dependencies: {};
        devDependencies: {
            typescript: string;
        };
        entry: string;
        main: string;
        environment: string;
        mode: string;
    };
};

/**
 * ------------------------ Public documentation ------------------------
 *
 * From this section on, all types are artificially created and maintained
 * for public documentation purposes. So, this might not reflect precisely
 * the component behavior, but it still needs to be reliable and clear for
 * general usage.
 */
/**
 * @category Components
 */
interface SandpackProps {
    /**
     * It accepts an object, where each key is the relative
     * path of that file in the sandbox folder structure. Files passed in
     * through the files prop override those in the template structure.
     *
     * Since each template uses the same type to define the files, you can
     * overwrite the contents of any of the template files.
     *
     * Example:
     * ```js
     * {
     *  "/App.js": "export default () => 'foo'"
     * }
     * ```
     */
    files?: Record<string, string> | SandpackFile;
    /**
     * Set of presets to easily initialize sandboxes. Each template contains
     * its files, environment and dependencies, and you can overwrite it
     * using `customSetup` or `dependencies`.
     */
    template?: SandpackPredefinedTemplate;
    /**
     * Pass custom properties to set your own Sandpack environment.
     *
     * Since each template uses the same type to define the files, you can
     * overwrite the contents of any of the template files.
     */
    customSetup?: SandpackSetup;
    /**
     * The theme specifies the color set of the components, syntax highlight,
     * and typography. Use this prop in order to match the design aspect of your website.
     *
     * Set as `auto` to turn it color scheme sensitive
     */
    theme?: SandpackThemeProp;
    /**
     * Pass custom properties to customize the interface and the behavior
     * of the sandbox, such as initialization mode, recompile mode, files resolver, etc.
     */
    options?: SandpackOptions;
}
/**
 * @category Setup
 */
interface SandpackOptions {
    /**
     * List of files that will be visible for the user interacts with.
     * It defaults to the main file from a given template.
     */
    visibleFiles?: string[];
    /**
     * Use this to set a file as active by default in the editor component.
     * It defaults to the main file from a given template.
     */
    activeFile?: string;
    editorWidthPercentage?: number;
    editorHeight?: CSSProperties['height'];
    classes?: Record<string, string>;
    showNavigator?: boolean;
    showLineNumbers?: boolean;
    showInlineErrors?: boolean;
    showRefreshButton?: boolean;
    showTabs?: boolean;
    showConsoleButton?: boolean;
    showConsole?: boolean;
    closableTabs?: boolean;
    wrapContent?: boolean;
    codeEditor?: SandpackCodeOptions;
    /**
     * This disables editing of content by the user in all files.
     */
    readOnly?: boolean;
    /**
     * Controls the visibility of Read-only label, which will only
     * appears when `readOnly` is `true`
     */
    showReadOnly?: boolean;
    /**
     * This provides a way to control how some components are going to
     * be initialized on the page. The CodeEditor and the Preview components
     * are quite expensive and might overload the memory usage, so this gives
     * a certain control of when to initialize them.
     */
    initMode?: SandpackInitMode;
    initModeObserverOptions?: IntersectionObserverInit;
    autorun?: boolean;
    recompileMode?: 'immediate' | 'delayed';
    recompileDelay?: number;
    /**
     * By default, Sandpack generates a random value to use as an id.
     * Use this to override this value if you need predictable values.
     */
    id?: string;
    logLevel?: SandpackLogLevel;
    bundlerURL?: string;
    startRoute?: string;
    skipEval?: boolean;
    fileResolver?: FileResolver;
    externalResources?: string[];
}
/**
 * @category Setup
 */
interface SandpackSetup {
    /**
     * Any template will include the needed dependencies,
     * but you can specify any additional dependencies. The key
     * should be the name of the package, while the value is the version,
     * in exactly the same format as it would be inside package.json.
     *
     * Examples:
     * ```js
     * {
     *  "react": "latest",
     *  "@material-ui/core": "4.12.3",
     * }
     * ```
     */
    dependencies?: Record<string, string>;
    /**
     * Sandpack doesn't install devDependencies, because most tools in there
     * were build tools, which is not necessary to properly run a sandbox,
     * but maybe required for running locally or export to CodeSandbox.
     *
     * Examples:
     * ```js
     * {
     *  "@types/react": "latest",
     * }
     * ```
     */
    devDependencies?: Record<string, string>;
    /**
     * The entry file is the starting point of the bundle process.
     *
     * If you change the path of the entry file, make sure you control
     * all the files that go into the bundle process, as prexisting
     * settings in the template might not work anymore.
     */
    entry?: string;
    /**
     * Each sandbox has its own bundler attached to them which are configured
     * to support a specific framework and emulate their official CLI tools.
     * They are not one-to-one implementations and thus do not support advanced
     * configuration like custom webpack configurations or ejecting. However,
     * they are designed to mirror the default behavior of the framework
     */
    environment?: SandboxEnvironment;
    /**
     * The custom private npm registry setting makes it possible
     * to retrieve npm packages from your own npm registry.
     *
     * Examples:
     * ```js
     * {
     *   enabledScopes: ["@codesandbox"],
     *   registryUrl: "//my-registry.domain.com",
     *   limitToScopes: true, // if false all packages will be fetched from custom registry
     *   registryAuthToken: "SECRET" // optinal value, if public
     * }
     * ```
     */
    npmRegistries?: Array<Omit<NpmRegistry, 'proxyEnabled'>>;
}
/**
 * @category Setup
 */
declare type SandboxEnvironment = 'angular-cli' | 'create-react-app' | 'create-react-app-typescript' | 'svelte' | 'parcel' | 'vue-cli' | 'static' | 'solid';
/**
 * @category Setup
 */
declare type SandpackPredefinedTemplate = 'angular' | 'react' | 'react-ts' | 'test-ts' | 'vanilla' | 'vanilla-ts' | 'vue' | 'vue3' | 'svelte' | 'solid';
/**
 * @category Setup
 */
interface SandpackFile {
    code: string;
    hidden?: boolean;
    active?: boolean;
    readOnly?: boolean;
}
/**
 * `immediate`: It immediately mounts all components, such as the code-editor
 * and the preview - this option might overload the memory usage
 * and resource from the browser on a page with multiple instances;
 *
 * `lazy`: Only initialize the components when the user is about to scroll
 * them to the viewport and keep these components mounted until the user
 * leaves the page - this is the default value;
 *
 * `user-visible`: Only initialize the components when the user is about
 * to scroll them to the viewport, but differently from lazy, this option
 * unmounts those components once it's no longer in the viewport.
 *
 * @category Setup
 */
declare type SandpackInitMode = 'immediate' | 'lazy' | 'user-visible';
/**
 * @category Theme
 */
declare type SandpackPredefinedTheme = 'light' | 'dark' | 'auto';
/**
 * @category Theme
 */
interface SandpackTheme {
    colors: {
        surface1: string;
        surface2: string;
        surface3: string;
        disabled: string;
        base: string;
        clickable: string;
        hover: string;
        accent: string;
        error?: string;
        errorSurface?: string;
        warning?: string;
        warningSurface?: string;
    };
    syntax: {
        plain: string | SandpackSyntaxStyle;
        comment: string | SandpackSyntaxStyle;
        keyword: string | SandpackSyntaxStyle;
        definition: string | SandpackSyntaxStyle;
        punctuation: string | SandpackSyntaxStyle;
        property: string | SandpackSyntaxStyle;
        tag: string | SandpackSyntaxStyle;
        static: string | SandpackSyntaxStyle;
        string?: string | SandpackSyntaxStyle;
    };
    font: {
        body: string;
        mono: string;
        size: string;
        lineHeight: string;
    };
}
/**
 * @category Theme
 */
interface SandpackSyntaxStyle {
    color?: string;
    fontStyle?: 'normal' | 'italic';
    fontWeight?: 'normal' | 'bold' | '100' | '200' | '300' | '400' | '500' | '600' | '700' | '800' | '900';
    textDecoration?: 'none' | 'underline' | 'line-through' | 'underline line-through';
}
/**
 * @category Theme
 */
declare type SandpackThemeProp = SandpackPredefinedTheme | DeepPartial<SandpackTheme>;
/**
 *
 * ------------------------ Internal types ---------------------------
 *
 * From this section on, all types are hidden because most of them are
 * strictly related to the internal typing system, and they are not
 * helpful for public documentation.
 *
 * For public purpose use SandpackProps instead.
 */
/**
 * @hidden
 */
declare type TemplateFiles<Name extends SandpackPredefinedTemplate> = keyof typeof SANDBOX_TEMPLATES[Name]['files'];
/**
 * @hidden
 */
interface SandpackInternal {
    <Files extends SandpackFiles | any, TemplateName extends SandpackPredefinedTemplate = 'vanilla'>(props: SandpackInternalProps<Files, TemplateName> & {
        files?: Files;
        template?: TemplateName;
    }): JSX.Element;
}
interface SandpackInternalProvider {
    <Files extends SandpackFiles, TemplateName extends SandpackPredefinedTemplate>(
    /**
     * Infer files & template values
     */
    props: SandpackProviderProps<Files, TemplateName> & {
        files?: Files;
        template?: TemplateName;
        children?: JSX.Element;
    }): SandpackProviderProps & SandpackProviderState;
}
/**
 * @hidden
 */
interface SandpackRootProps<Files extends SandpackFiles | any, TemplateName extends SandpackPredefinedTemplate> {
    files?: Files;
    template?: TemplateName;
    customSetup?: SandpackSetup;
    theme?: SandpackThemeProp;
    className?: string;
}
/**
 * @hidden
 */
interface SandpackInternalOptions<Files extends SandpackFiles | any = any, TemplateName extends SandpackPredefinedTemplate = SandpackPredefinedTemplate> {
    visibleFiles?: Array<Files extends SandpackFiles ? TemplateFiles<TemplateName> | keyof Files : TemplateFiles<TemplateName>>;
    activeFile?: Files extends SandpackFiles ? TemplateFiles<TemplateName> | keyof Files : TemplateFiles<TemplateName>;
    initMode?: SandpackInitMode;
    initModeObserverOptions?: IntersectionObserverInit;
    autorun?: boolean;
    recompileMode?: 'immediate' | 'delayed';
    recompileDelay?: number;
    id?: string;
    logLevel?: SandpackLogLevel;
    bundlerURL?: string;
    startRoute?: string;
    skipEval?: boolean;
    fileResolver?: FileResolver;
    externalResources?: string[];
    classes?: Record<string, string>;
}
interface SandpackInternalProps<Files extends SandpackFiles | any, TemplateName extends SandpackPredefinedTemplate> extends SandpackRootProps<Files, TemplateName> {
    options?: SandpackInternalOptions<Files, TemplateName> & {
        editorWidthPercentage?: number;
        editorHeight?: CSSProperties['height'];
        showNavigator?: boolean;
        showLineNumbers?: boolean;
        showInlineErrors?: boolean;
        showRefreshButton?: boolean;
        showTabs?: boolean;
        showConsoleButton?: boolean;
        showConsole?: boolean;
        closableTabs?: boolean;
        wrapContent?: boolean;
        codeEditor?: SandpackCodeOptions;
        /**
         * This disables editing of content by the user in all files.
         */
        readOnly?: boolean;
        /**
         * Controls the visibility of Read-only label, which will only
         * appears when `readOnly` is `true`
         */
        showReadOnly?: boolean;
    };
}
/**
 * @hidden
 */
interface SandpackProviderProps<Files extends SandpackFiles = SandpackFiles, TemplateName extends SandpackPredefinedTemplate = SandpackPredefinedTemplate> extends SandpackRootProps<Files, TemplateName>, HTMLAttributes {
    options?: SandpackInternalOptions<Files, TemplateName>;
    children?: JSX.Element;
}
/**
 * @hidden
 */
declare type SandpackClientDispatch = (msg: SandpackMessage, clientId?: string) => void;
/**
 * @hidden
 */
declare type SandpackClientListen = (listener: ListenerFunction, clientId?: string) => UnsubscribeFunction;
/**
 * @hidden
 */
declare type SandpackContext = SandpackState & {
    dispatch: SandpackClientDispatch;
    listen: SandpackClientListen;
};
/**
 * @hidden
 */
interface SandpackState {
    bundlerState: BundlerState | undefined;
    /**
     * List the file path listed in the file tab,
     * which will allow the user to interact with.
     */
    visibleFiles: string[];
    /**
     * List the file path listed in the file tab,
     *
     * Can only be changed through the properties of sandpackprovider (files/options)
     */
    visibleFilesFromProps: string[];
    /**
     * Path to the file will be open in the code editor
     * when the component mounts
     */
    activeFile: string;
    startRoute?: string;
    autorun?: boolean;
    /**
     * Returns the current state of the editor, meaning that any
     * changes from the original `files` must return a `dirty` value;
     * otherwise, it'll return `pristine`
     */
    editorState: EditorState;
    error: SandpackError | null;
    files: SandpackBundlerFiles;
    environment?: SandboxEnvironment;
    status: SandpackStatus;
    initMode: SandpackInitMode;
    clients: Record<string, SandpackClient>;
    runSandpack: () => void;
    registerBundler: (iframe: HTMLIFrameElement, clientId: string) => void;
    unregisterBundler: (clientId: string) => void;
    updateFile: (pathOrFiles: string | SandpackFiles, code?: string) => void;
    addFile: (pathOrFiles: string | SandpackFiles, code?: string) => void;
    updateCurrentFile: (newCode: string) => void;
    openFile: (path: string) => void;
    closeFile: (path: string) => void;
    deleteFile: (path: string) => void;
    setActiveFile: (path: string) => void;
    resetFile: (path: string) => void;
    resetAllFiles: () => void;
    registerReactDevTools: (value: ReactDevToolsMode) => void;
    /**
     * Element refs
     * Different components inside the SandpackProvider might register certain elements of interest for sandpack
     * eg: lazy anchor - if no component registers this, then the sandpack runs on mount, without lazy mode
     */
    lazyAnchorRef: Ref<HTMLDivElement>;
    /**
     * eg: error screen - if no component registers this, the bundler needs to show the custom error screen
     * When the value is boolean, we only care if the components have the responsibility to render the elements,
     * we don't need the actual element reference
     */
    errorScreenRegisteredRef: boolean;
    openInCSBRegisteredRef: boolean;
    loadingScreenRegisteredRef: boolean;
}
/**
 * @hidden
 */
declare type SandpackStatus = 'initial' | 'idle' | 'running' | 'timeout' | 'done';
/**
 * @hidden
 */
declare type EditorState = 'pristine' | 'dirty';
/**
 * @hidden
 */
interface SandboxTemplate {
    files: Record<string, SandpackFile>;
    dependencies: Record<string, string>;
    devDependencies?: Record<string, string>;
    entry: string;
    main: string;
    environment: SandboxEnvironment;
}
/**
 * @hidden
 */
declare type SandpackFiles = Record<string, string | SandpackFile>;
/**
 * Custom properties to be used in the SandpackCodeEditor component,
 * some of which are exclusive to customize the CodeMirror instance.
 * @hidden
 */
interface SandpackCodeOptions {
    /**
     * CodeMirror extensions for the editor state, which can
     * provide extra features and functionalities to the editor component.
     */
    extensions?: CodeEditorProps['extensions'];
    /**
     * Property to register CodeMirror extension keymap.
     */
    extensionsKeymap?: CodeEditorProps['extensionsKeymap'];
}
/**
 * @hidden
 */
declare type DeepPartial<Type> = {
    [Property in keyof Type]?: DeepPartial<Type[Property]>;
};
/**
 * @hidden
 */
interface FileResolver {
    isFile: (path: string) => Promise<boolean>;
    readFile: (path: string) => Promise<string>;
}
/**
 * @hidden
 */
interface SandpackProviderState {
    files: SandpackBundlerFiles;
    environment?: SandboxEnvironment;
    visibleFiles: Array<TemplateFiles<SandpackPredefinedTemplate> | string>;
    activeFile: TemplateFiles<SandpackPredefinedTemplate> | string;
    startRoute?: string;
    initMode: SandpackInitMode;
    bundlerState?: BundlerState;
    error: SandpackError | null;
    sandpackStatus: SandpackStatus;
    editorState: EditorState;
    reactDevTools?: ReactDevToolsMode;
}

/**
 * Sandpack component
 */
declare const Sandpack: SandpackInternal;

/**
 * code area
 */
declare const CodeMirror: DefineComponent<CodeMirrorProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<CodeMirrorProps>, {}>;

declare type Decorators = Array<{
    className?: string;
    line: number;
    startColumn?: number;
    endColumn?: number;
    elementAttributes?: Record<string, string>;
}>;
declare type FileType = 'js' | 'jsx' | 'ts' | 'tsx' | 'css' | 'scss' | 'less' | 'html' | 'vue' | 'markdown';
interface CodeMirrorRef {
    getCodemirror: () => EditorView | undefined;
}
interface CodeMirrorProps {
    onCodeUpdate?: (newCode: string) => void;
    id?: string;
    code: string;
    filePath?: string;
    fileType?: FileType;
    /**
     * @default false
     */
    showLineNumbers?: boolean;
    /**
     * @default false
     */
    showInlineErrors?: boolean;
    /**
     * @default false
     */
    wrapContent?: boolean;
    /**
     * @default 'pristine'
     */
    editorState?: EditorState;
    /**
     * This disables editing of content by the user in all files.
     * @default false
     */
    readOnly?: boolean;
    /**
     * Controls the visibility of Read-only label, which will only
     * appears when `readOnly` is `true`
     * @default true
     */
    showReadOnly?: boolean;
    /**
     * Provides a way to draw or style a piece of the content.
     */
    decorators?: Decorators;
    /**
     * @default 'lazy'
     */
    initMode: SandpackInitMode;
    /**
     * @default []
     */
    extensions?: Extension[];
    /**
     * @default []
     */
    extensionsKeymap?: KeyBinding[];
}
interface CodeEditorProps {
    style?: CSSProperties;
    showTabs?: boolean;
    showLineNumbers?: boolean;
    showInlineErrors?: boolean;
    showRunButton?: boolean;
    wrapContent?: boolean;
    closableTabs?: boolean;
    /**
     * This provides a way to control how some components are going to
     * be initialized on the page. The CodeEditor and the Preview components
     * are quite expensive and might overload the memory usage, so this gives
     * a certain control of when to initialize them.
     */
    initMode?: SandpackInitMode;
    /**
     * CodeMirror extensions for the editor state, which can
     * provide extra features and functionalities to the editor component.
     */
    extensions?: Extension[];
    /**
     * Property to register CodeMirror extension keymap.
     */
    extensionsKeymap?: KeyBinding[];
    /**
     * By default, Sandpack generates a random value to use as an id.
     * Use this to override this value if you need predictable values.
     */
    id?: string;
    /**
     * This disables editing of the editor content by the user.
     */
    readOnly?: boolean;
    /**
     * Controls the visibility of Read-only label, which will only
     * appears when `readOnly` is `true`
     */
    showReadOnly?: boolean;
}
declare const SandpackCodeEditor: DefineComponent<CodeEditorProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<CodeEditorProps>, {}>;

declare const tabButton: _stitches_core_types_styled_component.CssComponent<never, {}, {}, _stitches_core_types_css_util.CSS<{}, {}, _stitches_core_types_config.DefaultThemeMap, {}>>;
interface FileTabsProps {
    /**
     * This adds a close button next to each file with a unique trigger to close it.
     */
    closableTabs?: boolean;
}
/**
 * FileTabs is a list of all open files, the active file, and its state.
 */
declare const FileTabs: DefineComponent<FileTabsProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<FileTabsProps>, {}>;

interface Props$2 extends SandpackFileExplorerProp {
    prefixedPath: string;
    files: SandpackBundlerFiles;
    selectFile: (path: string) => void;
    activeFile: NonNullable<SandpackOptions['activeFile']>;
    depth: number;
    visibleFiles?: NonNullable<SandpackOptions['visibleFiles']>;
    autoHiddenFiles?: boolean;
}
/**
 * Directory
 */
declare const Directory: DefineComponent<Props$2, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<Props$2>, {}>;

interface Props$1 {
    path: string;
    selectFile?: (path: string) => void;
    active?: boolean;
    onClick?: (e: MouseEvent) => void;
    depth: number;
    isDirOpen?: boolean;
}
/**
 * File
 */
declare const File: DefineComponent<Props$1, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<Props$1>, {}>;

interface ModuleListProps extends SandpackFileExplorerProp {
    prefixedPath: string;
    files: SandpackBundlerFiles;
    selectFile: (path: string) => void;
    activeFile: NonNullable<SandpackOptions['activeFile']>;
    visibleFiles?: NonNullable<SandpackOptions['visibleFiles']>;
    depth?: number;
    autoHiddenFiles?: boolean;
}
/**
 * ModuleList
 */
declare const ModuleList: DefineComponent<ModuleListProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<ModuleListProps>, {}>;

interface SandpackFileExplorerProp {
    /**
     * enable auto hidden file in file explorer
     *
     * @description set with hidden property in files property
     * @default false
     */
    autoHiddenFiles?: boolean;
}
/**
 * SandpackFileExplorer
 */
declare const SandpackFileExplorer: DefineComponent<SandpackFileExplorerProp, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<SandpackFileExplorerProp>, {}>;

interface CodeViewerProps {
    showTabs?: boolean;
    showLineNumbers?: boolean;
    /**
     * Provides a way to draw or style a piece of the content.
     */
    decorators?: Decorators;
    code?: string;
    wrapContent?: boolean;
    /**
     * This provides a way to control how some components are going to
     * be initialized on the page. The CodeEditor and the Preview components
     * are quite expensive and might overload the memory usage, so this gives
     * a certain control of when to initialize them.
     */
    initMode?: SandpackInitMode;
}
declare const SandpackCodeViewer: DefineComponent<CodeViewerProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<CodeViewerProps>, {}>;

interface NavigatorProps {
    clientId: string;
    onURLChange?: (newURL: string) => void;
}
declare const Navigator: DefineComponent<NavigatorProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<NavigatorProps>, {}>;

interface RefreshButtonProps {
    clientId?: string;
}
declare const RefreshButton: DefineComponent<RefreshButtonProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<RefreshButtonProps>, {}>;

interface PreviewProps {
    showNavigator?: boolean;
    showOpenInCodeSandbox?: boolean;
    showRefreshButton?: boolean;
    showSandpackErrorOverlay?: boolean;
    actionsChildren?: JSX.Element;
}

interface SandpackPreviewRef {
    /**
     * Retrieve the current Sandpack client instance from preview
     */
    getClient: () => SandpackClient | null;
    /**
     * Returns the client id, which will be used to
     * initialize a client in the main Sandpack context
     */
    clientId: string;
}
declare const SandpackPreview: DefineComponent<PreviewProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<PreviewProps>, {}>;

declare const SandpackTranspiledCode: DefineComponent<CodeViewerProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<CodeViewerProps>, {}>;

declare type SandpackConsoleData = Array<{
    data: Array<string | Record<string, string>> | undefined;
    id: string;
    method: SandpackMessageConsoleMethods;
}>;

interface SandpackConsoleProps {
    clientId?: string;
    showHeader?: boolean;
    showSyntaxError?: boolean;
    maxMessageCount?: number;
    onLogsChange?: (logs: SandpackConsoleData) => void;
}
/**
 * `SandpackConsole` is a Sandpack devtool that allows printing
 * the console logs from a Sandpack client. It is designed to be
 * a light version of a browser console, which means that it's
 * limited to a set of common use cases you may encounter when coding.
 */
declare const SandpackConsole: DefineComponent<SandpackConsoleProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<SandpackConsoleProps>, {}>;

/**
 * It provides an interface to consume the logs from a sandpack client.
 *
 * @category Hooks
 */
declare const useSandpackConsole: (props?: {
    clientId?: string;
    maxMessageCount?: number;
    showSyntaxError?: boolean;
}) => {
    logs: Ref<SandpackConsoleData>;
    reset: () => void;
};

declare type TestStatus = 'idle' | 'running' | 'pass' | 'fail';
interface Test {
    name: string;
    blocks: string[];
    status: TestStatus;
    path: string;
    errors: TestError[];
    duration?: number | undefined;
}

interface Describe {
    name: string;
    tests: Record<string, Test>;
    describes: Record<string, Describe>;
}

declare type Spec = {
    error?: TestError;
} & Describe;

interface Props {
    verbose?: boolean;
    watchMode?: boolean;
    onComplete?: (specs: Record<string, Spec>) => void;
    actionsChildren?: JSX.Element;
}
declare const SandpackTests: DefineComponent<Props, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<Props>, {}>;

declare const layoutClassName: _stitches_core_types_styled_component.CssComponent<never, {}, {}, _stitches_core_types_css_util.CSS<{}, {}, _stitches_core_types_config.DefaultThemeMap, {}>>;
declare const SandpackLayout: vue.DefineComponent<{}, () => JSX.Element, {}, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{}>>, {}>;

declare const stackClassName: _stitches_core_types_styled_component.CssComponent<never, {}, {}, _stitches_core_types_css_util.CSS<{}, {}, _stitches_core_types_config.DefaultThemeMap, {}>>;
declare const SandpackStack: vue.DefineComponent<{}, () => JSX.Element, {}, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{}>>, {}>;

declare const ErrorOverlay: vue.DefineComponent<{}, () => JSX.Element, {}, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{}>>, {}>;

interface LoadingOverlayProps {
    clientId?: string;
    /**
     * It enforces keeping the loading state visible,
     * which is helpful for external loading states.
     */
    loading?: boolean;
    showOpenInCodeSandbox: boolean;
}
declare const LoadingOverlay: DefineComponent<LoadingOverlayProps, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<LoadingOverlayProps>, {}>;

declare const OpenInCodeSandboxButton: vue.DefineComponent<{}, () => JSX.Element, {}, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{}>>, {}>;

declare const UnstyledOpenInCodeSandboxButton: vue.DefineComponent<{}, () => JSX.Element, {}, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{}>>, {}>;

declare const RunButton: vue.DefineComponent<{
    onClick: {
        type: PropType<((event: MouseEvent) => void) | undefined>;
        default: null;
    };
}, () => JSX.Element, unknown, {}, {}, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<vue.ExtractPropTypes<{
    onClick: {
        type: PropType<((event: MouseEvent) => void) | undefined>;
        default: null;
    };
}>>, {
    onClick: ((event: MouseEvent) => void) | undefined;
}>;

/**
 * This returns the current state of the active file
 * and a method to update its content.
 *
 * @category Hooks
 */
declare const useActiveCode: () => {
    code: Ref<string>;
    readOnly: Ref<boolean>;
    updateCode: (newCode: string) => void;
};

/**
 * @category Hooks
 */
declare const useErrorMessage: () => ComputedRef<SandpackError>;

declare type LoadingOverlayState = 'LOADING' | 'PRE_FADING' | 'FADING' | 'HIDDEN' | 'TIMEOUT';
declare const FADE_ANIMATION_DURATION = 200;
interface LoadingOverlayStateProp {
    clientId?: string;
    /**
     * It enforces keeping the loading state visible,
     * which is helpful for external loading states.
     */
    loading?: boolean;
}
/**
 * @category Hooks
 */
declare const useLoadingOverlayState: (props: LoadingOverlayStateProp) => vue.Ref<LoadingOverlayState>;

/**
 * @category Hooks
 */
declare const useSandpackNavigation: (clientId?: string) => {
    refresh: () => void;
    back: () => void;
    forward: () => void;
};

interface SandpackThemeProviderContext {
    theme: SandpackTheme;
    id: string;
    mode: 'dark' | 'light';
}
interface IProp extends HTMLAttributes {
    theme?: SandpackThemeProp;
    mode?: 'dark' | 'light';
}
declare const SandpackThemeContext: InjectionKey<SandpackThemeProviderContext>;
declare const SandpackThemeProvider: DefineComponent<IProp, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<IProp>, {
    class?: any;
    inlist?: any;
}>;

/**
 * useSandpackTheme
 */
declare const useSandpackTheme: () => SandpackThemeProviderContext;

declare type ContextType<T> = any;
interface CreateContext<T> {
    Provider: DefineComponent<{}, () => VNode | VNode[] | undefined, any>;
    state: UnwrapRef<T> | T;
}
declare const createContext: <T extends Record<string, any>>(context: any, contextInjectKey?: InjectionKey<any>) => CreateContext<T>;
declare const useContext: <T>(contextInjectKey?: InjectionKey<any>, defaultValue?: any) => T;

interface UseSandpackClient {
    sandpack: SandpackState;
    getClient: () => SandpackClient | null;
    iframe: Ref<HTMLIFrameElement | null>;
    listen: (listener: ListenerFunction) => UnsubscribeFunction;
    dispatch: (message: SandpackMessage) => void;
    clientId: Ref<string>;
}
/**
 * It registers a new sandpack client and returns its instance,
 * listeners, and dispatch function. Using it when creating a custom
 * component to interact directly with the client is recommended.
 * For other cases, use `useSandpack` instead.
 *
 * @category Hooks
 */
declare const useSandpackClient: () => UseSandpackClient;

interface UseSandpack {
    sandpack: SandpackState;
    dispatch: SandpackClientDispatch;
    listen: SandpackClientListen;
}
declare const SandpackStateContext: InjectionKey<UnwrapNestedRefs<SandpackState>>;
/**
 * SandpackProvider
 * Main context provider that should wraps your entire component.
 * Use * [`useSandpack`](/api/react/#usesandpack) hook, which gives you the entire context object to play with.
 */
declare const SandpackProvider: DefineComponent<SandpackProviderProps<SandpackFiles, SandpackPredefinedTemplate>, {}, {}, vue.ComputedOptions, vue.MethodOptions, vue.ComponentOptionsMixin, vue.ComponentOptionsMixin, {}, string, vue.VNodeProps & vue.AllowedComponentProps & vue.ComponentCustomProps, Readonly<SandpackProviderProps<SandpackFiles, SandpackPredefinedTemplate>>, {
    class?: any;
    inlist?: any;
}>;
/**
 * useSandpack
 */
declare function useSandpack(): UseSandpack;

/**
 * @category Theme
 */
declare const defaultLight: SandpackTheme;
/**
 * @category Theme
 */
declare const defaultDark: SandpackTheme;
/**
 * @category Theme
 */
declare const SANDPACK_THEMES: Record<SandpackPredefinedTheme, SandpackTheme>;

declare const RunIcon: () => JSX.Element;
declare const BackwardIcon: () => JSX.Element;
declare const ForwardIcon: () => JSX.Element;
declare const RefreshIcon: () => JSX.Element;
declare const CleanIcon: () => JSX.Element;
declare const ExportIcon: () => JSX.Element;
declare const DirectoryIconOpen: () => JSX.Element;
declare const DirectoryIconClosed: () => JSX.Element;
declare const FileIcon: () => JSX.Element;
declare const CloseIcon: () => JSX.Element;
declare const ConsoleIcon: () => JSX.Element;

declare const getCssText: () => string;

export { ANGULAR_TEMPLATE, BackwardIcon, CleanIcon, CloseIcon, CodeMirror as CodeEditor, CodeEditorProps, CodeMirrorProps, CodeMirrorRef, CodeViewerProps, ConsoleIcon, ContextType, CreateContext, Decorators, DeepPartial, Directory, DirectoryIconClosed, DirectoryIconOpen, EditorState, ErrorOverlay, ExportIcon, FADE_ANIMATION_DURATION, File, FileIcon, FileResolver, FileTabs, FileTabsProps, FileType, ForwardIcon, LoadingOverlay, LoadingOverlayProps, LoadingOverlayState, ModuleList, Navigator, OpenInCodeSandboxButton, PreviewProps, REACT_TEMPLATE, REACT_TYPESCRIPT_TEMPLATE, RefreshButton, RefreshIcon, RunButton, RunIcon, SANDBOX_TEMPLATES, SANDPACK_THEMES, SOLID_TEMPLATE, SVELTE_TEMPLATE, SandboxEnvironment, SandboxTemplate, Sandpack, SandpackClientDispatch, SandpackClientListen, SandpackCodeEditor, SandpackCodeOptions, SandpackCodeViewer, SandpackConsole, SandpackContext, SandpackFile, SandpackFileExplorer, SandpackFileExplorerProp, SandpackFiles, SandpackInitMode, SandpackInternal, SandpackInternalOptions, SandpackInternalProps, SandpackInternalProvider, SandpackLayout, SandpackOptions, SandpackPredefinedTemplate, SandpackPredefinedTheme, SandpackPreview, SandpackPreviewRef, SandpackProps, SandpackProvider, SandpackProviderProps, SandpackProviderState, SandpackSetup, SandpackStack, SandpackState, SandpackStateContext, SandpackStatus, SandpackTests, SandpackTheme, SandpackThemeContext, SandpackThemeProp, SandpackThemeProvider, SandpackThemeProviderContext, SandpackTranspiledCode, TEST_TYPESCRIPT_TEMPLATE, TemplateFiles, UnstyledOpenInCodeSandboxButton, UseSandpack, VANILLA_TEMPLATE, VANILLA_TYPESCRIPT_TEMPLATE, VUE_TEMPLATE, VUE_TEMPLATE_3, createContext, SanpackPlugin as default, defaultDark, defaultLight, getCssText as getSandpackCssText, layoutClassName, stackClassName, tabButton, useActiveCode, useContext, useErrorMessage, useLoadingOverlayState, useSandpack, useSandpackClient, useSandpackConsole, useSandpackNavigation, useSandpackTheme };
